﻿using System;
using System.IO;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using TemplateMethodTcpServer;

namespace HttpServer
{
    public class HttpServer : AbstractTcpServer
    {
        private static readonly string RootCatalog = "c:/temp";
        public HttpServer(int port) : base(port) { }

        protected override void HandleRequest(TcpClient tcpClient)
        {
            try
            {
                NetworkStream clientStream = tcpClient.GetStream();
                using (StreamReader fromClient = new StreamReader(clientStream, Encoding.UTF8))
                using (StreamWriter toClient = new StreamWriter(clientStream, Encoding.UTF8))
                {
                    String requestLine = fromClient.ReadLine();
                    Console.WriteLine("Request line: " + requestLine.Length + " " + requestLine);
                    if (String.IsNullOrWhiteSpace(requestLine))
                    {
                        SendError(toClient, 400, "Bad Request");
                    }
                    else
                    {
                        string[] parts = requestLine.Split(new[] { ' ' });
                        if (parts.Count() != 3)
                        {
                            SendError(toClient, 400, "Bad Request");
                        }
                        else
                        {
                            String method = parts[0];
                            String uri = parts[1];
                            String protocol = parts[2];
                            if (!IsProtocolLegal(protocol))
                            {
                                SendError(toClient, 400, "Illegal protocol");
                            }
                            else
                            {
                                switch (method)
                                {
                                    case "GET": DoGet(uri, toClient);
                                        break;
                                    case "POST":
                                    case "HEAD": SendError(toClient, 501, "Not Implemented");
                                        break;
                                    default: SendError(toClient, 400, "Bad Request");
                                        break;
                                }
                            }
                        }
                    }
                    toClient.Flush();
                }
            }
            finally { tcpClient.Close(); }
        }

        private static void DoGet(string uri, StreamWriter toClient)
        {
            try
            {
                string fullFilename = RootCatalog + uri;
                using (FileStream fileStream = new FileStream(fullFilename, FileMode.Open, FileAccess.Read))
                {
                    FileInfo fileInfo = new FileInfo(fullFilename);
                    long sizeInBytes = fileInfo.Length;
                    toClient.Write("HTTP/1.0 200 OK\r\n");
                    toClient.Write("Content-Length: {0}\r\n", sizeInBytes);
                    toClient.Write("\r\n");
                    toClient.Flush();
                    fileStream.CopyTo(toClient.BaseStream);
                }
            }
            catch (FileNotFoundException) { SendError(toClient, 404, "Not Found"); }
        }

        private static bool IsProtocolLegal(String protocol)
        {
            return "HTTP/1.1".Equals(protocol) || "HTTP/1.0".Equals(protocol);
        }

        private static void SendError(TextWriter toClient, int statusCode, string reasonPhrase)
        {
            String responseLine = string.Format("HTTP/1.0 {0} {1}\r\n", statusCode, reasonPhrase);
            toClient.Write(responseLine);
            toClient.Write("Content-Type: text/html\r\n");
            toClient.Write("\r\n");
            toClient.Write("<html><head><title>{0} {1}</title></head><body>{0} {1}</body></html>",
                 statusCode, reasonPhrase);
        }
    }
}
