﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Threading.Tasks;

namespace TemplateMethodTcpServer
{
    public abstract class AbstractTcpServer
    {
        public int Port { get; private set; }
        private bool _keepOnRunning = true;
        protected AbstractTcpServer(int port)
        {
            if (port < 0 || port > 65535)
            {
                throw new ArgumentOutOfRangeException("port", port, "port number out of range");
            }
            Port = port;
        }

        // Template method
        public void Run()
        {
            IPAddress address = IPAddress.Parse("127.0.0.1");
            TcpListener tcpListener = new TcpListener(address, Port);
            Console.WriteLine("Server ready. Listening on port: " + Port);
            tcpListener.Start();
            while (_keepOnRunning)
            {
                try
                {
                    while (!tcpListener.Pending())
                    {
                        Thread.Sleep(100);
                    }
                    TcpClient client = tcpListener.AcceptTcpClient();
                    Task.Run(() => HandleRequest(client));
                }
                catch (SocketException ex)
                {
                    Console.WriteLine("Serious problem: " + ex.Message);
                }
            }
            tcpListener.Stop();
        }

        protected abstract void HandleRequest(TcpClient client);

        public void Stop()
        {
            _keepOnRunning = false;
        }
    }
}
